﻿using ProjectSportCar;

namespace ProjectByldozer;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningByldozer
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityByldozer? EntityByldozer { get; private set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки бульдозера
    /// </summary>
    private int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки  бульдозера
    /// </summary>
    private int? _startPosY;

    /// <summary>
    /// Ширина прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningByldozerWidth = 150;

    /// <summary>
    /// Высота прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningByldozerHeight = 80;

    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="dump">Признак наличия отвала</param>
    /// <param name="bakingpowder">Признак наличия разрыхлитель</param>
    /// <param name="pipe">Признак наличия трубы</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool dump, bool bakingpowder, bool pipe)
    {
        EntityByldozer = new EntityByldozer();
        EntityByldozer.Init(speed, weight, bodyColor, additionalColor, dump, bakingpowder, pipe);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (_drawningByldozerWidth < width && _drawningByldozerHeight < height)
        {
            // TODO проверка, что объект "влезает" в размеры поля
            // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningByldozerWidth <= _pictureWidth) 
                {
                    _startPosX = 0;
                    
                }
                if (_startPosY + _drawningByldozerHeight <= _pictureHeight)
                {
                    _startPosY = 0;
                }
            }
            return true;
        }
        return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // Проверка выхода за границы формы по оси X 
        if (x < 0) // Если новая позиция находится левее левой границы формы 
        {
            x = 0; // Устанавливаем позицию по оси X равной нулю (левой границе формы) 
        }
        else if (x + _drawningByldozerWidth > _pictureWidth.Value) // Если новая позиция выходит за правую границу формы 
        {
            x = _pictureWidth.Value - _drawningByldozerWidth; // Корректируем позицию по оси X 
        }

        // Проверка выхода за границы формы по оси Y 
        if (y < 0) // Если новая позиция выше верхней границы формы 
        {
            y = 0; // Устанавливаем позицию по оси Y равной нулю (верхней границе формы) 
        }
        else if (y + _drawningByldozerHeight > _pictureHeight.Value) // Если новая позиция выходит за нижнюю границу формы 
        {
            y = _pictureHeight.Value - _drawningByldozerHeight; // Корректируем позицию по оси Y 
        }


        _startPosX = x;
        _startPosY = y;
    }





    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityByldozer == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value -EntityByldozer.Step > 0)
                {
                    _startPosX -= (int)EntityByldozer.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityByldozer.Step > 0)
                {
                    _startPosY -= (int)EntityByldozer.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningByldozerWidth + EntityByldozer.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityByldozer.Step;
                }
                return true;

            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningByldozerHeight + EntityByldozer.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityByldozer.Step;
                }

                return true;
            default:
                return false;
        }
    }


    ///<summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name = "g" ></ param >
    public void DrawTransport(Graphics g)
    {
        if (EntityByldozer == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }



        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityByldozer.AdditionalColor);
        Brush bodybrush = new SolidBrush(EntityByldozer.BodyColor);

        //границы бульдозера и колес
        g.DrawRectangle(pen, _startPosX.Value+30, _startPosY.Value + 20, 100, 30);
        g.DrawRectangle(pen, _startPosX.Value + 100, _startPosY.Value, 30, 20);

        g.DrawArc(pen, _startPosX.Value+30, _startPosY.Value + 45, 30, 40, 130, 100);
        g.DrawArc(pen, _startPosX.Value + 103, _startPosY.Value + 50, 30, 40, 258, 140);

        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 80, _startPosX.Value + 130, _startPosY.Value + 80);

        g.DrawEllipse(pen, _startPosX.Value + 60, _startPosY.Value + 67, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 78, _startPosY.Value + 67, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 95, _startPosY.Value + 67, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 70, _startPosY.Value + 53, 7, 7);
        g.DrawEllipse(pen, _startPosX.Value + 90, _startPosY.Value + 53, 7, 7);
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 55, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 110   , _startPosY.Value + 55, 20, 20);

        //заливка кабины и гусениц
        Brush br = new SolidBrush(Color.Black);
        g.FillRectangle(bodybrush, _startPosX.Value + 30, _startPosY.Value + 20, 100, 30);
        g.FillRectangle(bodybrush, _startPosX.Value + 100, _startPosY.Value, 30, 20);
        g.FillEllipse(br, _startPosX.Value + 35, _startPosY.Value + 55, 20, 20);
        g.FillEllipse(br, _startPosX.Value + 110, _startPosY.Value + 55, 20, 20);
        g.FillEllipse(br, _startPosX.Value + 60, _startPosY.Value + 67, 10, 10);
        g.FillEllipse(br, _startPosX.Value + 78, _startPosY.Value + 67, 10, 10);
        g.FillEllipse(br, _startPosX.Value + 95, _startPosY.Value + 67, 10, 10);
        g.FillEllipse(br, _startPosX.Value + 70, _startPosY.Value + 53, 7, 7);
        g.FillEllipse(br, _startPosX.Value + 90, _startPosY.Value + 53, 7, 7);

        //окно
        Brush blfara = new SolidBrush(Color.Blue);
        g.FillRectangle(blfara, _startPosX.Value + 110, _startPosY.Value + 5, 10, 25);

        ///труба
        if (EntityByldozer.Pipe)
        {
            g.DrawRectangle(pen, _startPosX.Value + 60, _startPosY.Value, 5, 20);
            g.FillRectangle(bodybrush, _startPosX.Value + 60, _startPosY.Value, 5, 20);
        }
        // отвал
        if (EntityByldozer.Dump)

        {
            Point point1 = new Point(_startPosX.Value+30, _startPosY.Value + 35);
            Point point2 = new Point(_startPosX.Value +25, _startPosY.Value + 35);
            Point point4 = new Point(_startPosX.Value +25, _startPosY.Value + 23);
            Point point5 = new Point(_startPosX.Value +20, _startPosY.Value + 23);
            Point point6 = new Point(_startPosX.Value +15, _startPosY.Value + 60);
            Point point7 = new Point(_startPosX.Value +2, _startPosY.Value + 80);
            Point point8 = new Point(_startPosX.Value + 25, _startPosY.Value + 80);
            Point point9 = new Point(_startPosX.Value + 25, _startPosY.Value + 46);
            Point point10 = new Point(_startPosX.Value+30, _startPosY.Value + 46);

            Point[] Dump = { point1, point2, point4, point5, point6, point7, point8, point9, point10 };
            g.FillPolygon(additionalBrush, Dump);
        }

        if (EntityByldozer.Bakingpowder)

        {
            Point point1 = new Point(_startPosX.Value + 131, _startPosY.Value + 25);
            Point point2 = new Point(_startPosX.Value + 150, _startPosY.Value + 25);
            Point point4 = new Point(_startPosX.Value + 150, _startPosY.Value + 35);
            Point point5 = new Point(_startPosX.Value + 147, _startPosY.Value + 35);
            Point point6 = new Point(_startPosX.Value + 147, _startPosY.Value + 60);
            Point point7 = new Point(_startPosX.Value + 144, _startPosY.Value + 60);
            Point point8 = new Point(_startPosX.Value + 145, _startPosY.Value + 80);
            Point point9 = new Point(_startPosX.Value + 137, _startPosY.Value + 60);
            Point point10 = new Point(_startPosX.Value + 134, _startPosY.Value + 60);
            Point point11 = new Point(_startPosX.Value + 134, _startPosY.Value + 35);

            Point point12 = new Point(_startPosX.Value + 131, _startPosY.Value + 35);


            Point[] Bakingpowder = { point1, point2, point4, point5, point6, point7, point8, point9, point10, point11, point12 };
            g.FillPolygon(additionalBrush, Bakingpowder);

        }
    }
}
